@@ -23,6 +23,7 @@
 #endif
 
 #include <gst/gl/gl.h>
+#include <gst/gl/gstglfuncs.h>
 #if GST_GL_HAVE_PLATFORM_EGL && GST_GL_HAVE_DMABUF
 #include <gst/gl/egl/gsteglimage.h>
 #include <gst/allocators/gstdmabuf.h>
@@ -30,6 +31,7 @@
 
 #include "gstglelements.h"
 #include "gstgldownloadelement.h"
+#include "gstglutils.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_gl_download_element_debug);
 #define GST_CAT_DEFAULT gst_gl_download_element_debug
@@ -1239,7 +1241,7 @@ gst_gl_download_element_prepare_output_buffer (GstBaseTransform * bt,
   in_sync_meta = gst_buffer_get_gl_sync_meta (inbuf);
   if (in_sync_meta) {
     if (context) {
-      gst_gl_sync_meta_wait (in_sync_meta, context);
+      gst_gl_sync_meta_wait_cpu (in_sync_meta, context);
     } else if (dl->mode != GST_GL_DOWNLOAD_MODE_PASSTHROUGH) {
       GST_WARNING_OBJECT (dl, "No configured GL context in non-passthrough "
           "mode. Cannot wait on incoming `GstGLSyncMeta`");
@@ -1352,13 +1354,31 @@ static gboolean
 gst_gl_download_element_transform_meta (GstBaseTransform * bt,
     GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf)
 {
+  const GstMetaInfo *info = meta->info;
+  gboolean should_copy = TRUE;
+  const gchar *valid_tags[] = {
+    GST_META_TAG_VIDEO_STR,
+    GST_META_TAG_VIDEO_ORIENTATION_STR,
+    GST_META_TAG_VIDEO_SIZE_STR,
+    GST_META_TAG_VIDEO_COLORSPACE_STR,
+    NULL
+  };
+
   if (g_type_is_a (meta->info->api, GST_GL_SYNC_META_API_TYPE)) {
     GST_LOG_OBJECT (bt, "not copying GstGLSyncMeta onto output buffer");
     return FALSE;
   }
 
-  return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (bt, outbuf,
-      meta, inbuf);
+  should_copy = gst_meta_api_type_tags_contain_only (info->api, valid_tags);
+
+  /* Cant handle the tags in this meta, let the parent class handle it */
+  if (!should_copy) {
+    return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (bt,
+        outbuf, meta, inbuf);
+  }
+
+  /* No need to transform, we can safely copy this meta */
+  return TRUE;
 }
 
 static gboolean
@@ -1454,13 +1474,15 @@ gst_gl_download_element_propose_allocation (GstBaseTransform * bt,
   size = info.size;
   gst_buffer_pool_config_set_params (config, caps, size, 0, 0);
   gst_buffer_pool_config_set_gl_min_free_queue_size (config, 1);
-  gst_buffer_pool_config_add_option (config,
-      GST_BUFFER_POOL_OPTION_GL_SYNC_META);
 
   if (!gst_buffer_pool_set_config (pool, config)) {
     gst_object_unref (pool);
     goto config_failed;
   }
+
+  if (context->gl_vtable->FenceSync)
+    gst_query_add_allocation_meta (query, GST_GL_SYNC_META_API_TYPE, NULL);
+
   gst_query_add_allocation_pool (query, pool, size, 1, 0);
 
   gst_object_unref (pool);
