@@ -72,7 +72,7 @@
  * No special opengl extension is used in this pipeline, that's why it should work
  * with OpenGL >= 1.1. That's the case if you are using the MESA3D driver v1.3.
  * |[
- * gst-plugins-bas/tests/examples/gl/generic/cube
+ * gst-plugins-base/tests/examples/gl/generic/cube
  * ]| The graphic FPS scene can be greater than the input video FPS.
  * The graphic scene can be written from a client code through the
  * two glfilterapp properties.
@@ -595,21 +595,22 @@ gst_glimage_sink_set_rotate_method (GstGLImageSink * gl_sink,
 }
 
 static void
-gst_glimage_sink_navigation_send_event (GstNavigation * navigation, GstStructure
-    * structure)
+gst_glimage_sink_navigation_send_event (GstNavigation * navigation,
+    GstEvent * event)
 {
   GstGLImageSink *sink = GST_GLIMAGE_SINK (navigation);
   gboolean handled = FALSE;
-  GstEvent *event = NULL;
   GstGLWindow *window;
   guint width, height;
   gdouble x, y;
 
   if (!sink->context) {
-    gst_structure_free (structure);
+    gst_event_unref (event);
     return;
   }
 
+  event = gst_event_make_writable (event);
+
   window = gst_gl_context_get_window (sink->context);
   g_return_if_fail (GST_IS_GL_WINDOW (window));
 
@@ -618,47 +619,45 @@ gst_glimage_sink_navigation_send_event (GstNavigation * navigation, GstStructure
   gst_gl_window_get_surface_dimensions (window, &width, &height);
 
   /* Converting pointer coordinates to the non scaled geometry */
-  if (width != 0 && gst_structure_get_double (structure, "pointer_x", &x)
-      && height != 0 && gst_structure_get_double (structure, "pointer_y", &y)) {
+  if (width != 0 && height != 0 &&
+      gst_navigation_event_get_coordinates (event, &x, &y)) {
     gdouble stream_x, stream_y;
 
     _display_size_to_stream_size (sink, x, y, &stream_x, &stream_y);
 
-    gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE,
-        stream_x, "pointer_y", G_TYPE_DOUBLE, stream_y, NULL);
+    gst_navigation_event_set_coordinates (event, stream_x, stream_y);
   }
 
   /* Converting pointer scroll coordinates to the non scaled geometry */
-  if (width != 0 && gst_structure_get_double (structure, "delta_pointer_x", &x)
-      && height != 0
-      && gst_structure_get_double (structure, "delta_pointer_y", &y)) {
-    gdouble stream_x, stream_y;
+  if (width != 0 && height != 0 && gst_navigation_event_get_type (event)
+      == GST_NAVIGATION_EVENT_MOUSE_SCROLL) {
+    gdouble dx, dy, stream_dx, stream_dy;
 
-    _display_scroll_value_to_stream_scroll_value (sink, x, y, &stream_x,
-        &stream_y);
+    gst_navigation_event_parse_mouse_scroll_event (event, &x, &y, &dx, &dy);
+    _display_scroll_value_to_stream_scroll_value (sink, dx, dy, &stream_dx,
+        &stream_dy);
 
-    gst_structure_set (structure, "delta_pointer_x", G_TYPE_DOUBLE,
-        stream_x, "delta_pointer_y", G_TYPE_DOUBLE, stream_y, NULL);
+    /* FIXME: Add support for NavigationModifier */
+    gst_event_replace (&event,
+        gst_navigation_event_new_mouse_scroll (x, y, stream_dx, stream_dy,
+            GST_NAVIGATION_MODIFIER_NONE));
   }
 
-  event = gst_event_new_navigation (structure);
-  if (event) {
-    gst_event_ref (event);
-    handled = gst_pad_push_event (GST_VIDEO_SINK_PAD (sink), event);
+  gst_event_ref (event);
+  handled = gst_pad_push_event (GST_VIDEO_SINK_PAD (sink), event);
 
-    if (!handled)
-      gst_element_post_message ((GstElement *) sink,
-          gst_navigation_message_new_event ((GstObject *) sink, event));
+  if (!handled)
+    gst_element_post_message ((GstElement *) sink,
+        gst_navigation_message_new_event ((GstObject *) sink, event));
 
-    gst_event_unref (event);
-  }
+  gst_event_unref (event);
   gst_object_unref (window);
 }
 
 static void
 gst_glimage_sink_navigation_interface_init (GstNavigationInterface * iface)
 {
-  iface->send_event = gst_glimage_sink_navigation_send_event;
+  iface->send_event_simple = gst_glimage_sink_navigation_send_event;
 }
 
 #define gst_glimage_sink_parent_class parent_class
@@ -815,6 +814,7 @@ gst_glimage_sink_init (GstGLImageSink * glimage_sink)
   glimage_sink->handle_events = TRUE;
   glimage_sink->ignore_alpha = TRUE;
   glimage_sink->overlay_compositor = NULL;
+  glimage_sink->need_resize_window = FALSE;
 
   glimage_sink->mview_output_mode = DEFAULT_MULTIVIEW_MODE;
   glimage_sink->mview_output_flags = DEFAULT_MULTIVIEW_FLAGS;
@@ -946,18 +946,48 @@ static void
 gst_glimage_sink_key_event_cb (GstGLWindow * window, char *event_name, char
     *key_string, GstGLImageSink * gl_sink)
 {
+  GstEvent *event = NULL;
+
   GST_DEBUG_OBJECT (gl_sink, "event %s key %s pressed", event_name, key_string);
-  gst_navigation_send_key_event (GST_NAVIGATION (gl_sink),
-      event_name, key_string);
+  // FIXME: Add support for modifiers
+  if (0 == g_strcmp0 ("key-press", event_name)) {
+    event =
+        gst_navigation_event_new_key_press (key_string,
+        GST_NAVIGATION_MODIFIER_NONE);
+  } else if (0 == g_strcmp0 ("key-release", event_name)) {
+    event =
+        gst_navigation_event_new_key_release (key_string,
+        GST_NAVIGATION_MODIFIER_NONE);
+  }
+
+  if (event)
+    gst_navigation_send_event_simple (GST_NAVIGATION (gl_sink), event);
 }
 
 static void
 gst_glimage_sink_mouse_event_cb (GstGLWindow * window, char *event_name,
     int button, double posx, double posy, GstGLImageSink * gl_sink)
 {
+  GstEvent *event = NULL;
+
   GST_DEBUG_OBJECT (gl_sink, "event %s at %g, %g", event_name, posx, posy);
-  gst_navigation_send_mouse_event (GST_NAVIGATION (gl_sink),
-      event_name, button, posx, posy);
+  // FIXME: Add support for modifiers
+  if (0 == g_strcmp0 ("mouse-button-press", event_name)) {
+    event =
+        gst_navigation_event_new_mouse_button_press (button, posx, posy,
+        GST_NAVIGATION_MODIFIER_NONE);
+  } else if (0 == g_strcmp0 ("mouse-button-release", event_name)) {
+    event =
+        gst_navigation_event_new_mouse_button_release (button, posx, posy,
+        GST_NAVIGATION_MODIFIER_NONE);
+  } else if (0 == g_strcmp0 ("mouse-move", event_name)) {
+    event =
+        gst_navigation_event_new_mouse_move (posx, posy,
+        GST_NAVIGATION_MODIFIER_NONE);
+  }
+
+  if (event)
+    gst_navigation_send_event_simple (GST_NAVIGATION (gl_sink), event);
 }
 
 
@@ -967,19 +997,25 @@ gst_glimage_sink_mouse_scroll_event_cb (GstGLWindow * window,
     GstGLImageSink * gl_sink)
 {
   GST_DEBUG_OBJECT (gl_sink, "event scroll at %g, %g", posx, posy);
-  gst_navigation_send_mouse_scroll_event (GST_NAVIGATION (gl_sink),
-      posx, posy, delta_x, delta_y);
+  gst_navigation_send_event_simple (GST_NAVIGATION (gl_sink),
+      gst_navigation_event_new_mouse_scroll (posx, posy, delta_x, delta_y,
+          GST_NAVIGATION_MODIFIER_NONE));
 }
 
 static void
 _set_context (GstGLImageSink * gl_sink, GstGLContext * context)
 {
-  GST_GLIMAGE_SINK_LOCK (gl_sink);
-  if (gl_sink->context)
-    gst_object_unref (gl_sink->context);
+  GstGLContext *old_context;
 
+  GST_GLIMAGE_SINK_LOCK (gl_sink);
+
+  old_context = gl_sink->context;
   gl_sink->context = context;
+
   GST_GLIMAGE_SINK_UNLOCK (gl_sink);
+
+  if (old_context)
+    gst_object_unref (old_context);
 }
 
 static void
@@ -1492,11 +1528,19 @@ update_output_format (GstGLImageSink * glimage_sink)
   GstVideoMultiviewMode mv_mode;
   GstGLWindow *window = NULL;
   GstGLTextureTarget previous_target;
+  gint pre_width = 0, pre_height = 0;
+  gint cur_width = 0, cur_height = 0;
   GstStructure *s;
   const gchar *target_str;
   GstCaps *out_caps;
   gboolean ret;
 
+  pre_width = GST_VIDEO_INFO_WIDTH (out_info);
+  pre_height = GST_VIDEO_INFO_HEIGHT (out_info);
+
+  cur_width = GST_VIDEO_INFO_WIDTH (&glimage_sink->in_info);
+  cur_height = GST_VIDEO_INFO_HEIGHT (&glimage_sink->in_info);
+
   *out_info = glimage_sink->in_info;
   previous_target = glimage_sink->texture_target;
 
@@ -1590,6 +1634,10 @@ update_output_format (GstGLImageSink * glimage_sink)
     gst_caps_unref (glimage_sink->out_caps);
   glimage_sink->out_caps = out_caps;
 
+  if ((pre_width != 0 && pre_width != cur_width) ||
+      (pre_height != 0 && pre_height != cur_height))
+    glimage_sink->need_resize_window = TRUE;
+
   if (previous_target != GST_GL_TEXTURE_TARGET_NONE &&
       glimage_sink->texture_target != previous_target) {
     /* regenerate the shader for the changed target */
@@ -2450,25 +2498,30 @@ gst_glimage_sink_on_draw (GstGLImageSink * gl_sink)
 static void
 gst_glimage_sink_on_close (GstGLImageSink * gl_sink)
 {
-  GstGLWindow *window;
+  GstGLWindow *window = NULL;
 
   GST_WARNING_OBJECT (gl_sink, "Output window was closed");
 
-  window = gst_gl_context_get_window (gl_sink->context);
+  GST_GLIMAGE_SINK_LOCK (gl_sink);
+  if (gl_sink->context)
+    window = gst_gl_context_get_window (gl_sink->context);
+  GST_GLIMAGE_SINK_UNLOCK (gl_sink);
 
-  if (gl_sink->key_sig_id)
-    g_signal_handler_disconnect (window, gl_sink->key_sig_id);
-  gl_sink->key_sig_id = 0;
-  if (gl_sink->mouse_sig_id)
-    g_signal_handler_disconnect (window, gl_sink->mouse_sig_id);
-  gl_sink->mouse_sig_id = 0;
-  if (gl_sink->mouse_scroll_sig_id)
-    g_signal_handler_disconnect (window, gl_sink->mouse_scroll_sig_id);
-  gl_sink->mouse_scroll_sig_id = 0;
+  if (window) {
+    if (gl_sink->key_sig_id)
+      g_signal_handler_disconnect (window, gl_sink->key_sig_id);
+    gl_sink->key_sig_id = 0;
+    if (gl_sink->mouse_sig_id)
+      g_signal_handler_disconnect (window, gl_sink->mouse_sig_id);
+    gl_sink->mouse_sig_id = 0;
+    if (gl_sink->mouse_scroll_sig_id)
+      g_signal_handler_disconnect (window, gl_sink->mouse_scroll_sig_id);
+    gl_sink->mouse_scroll_sig_id = 0;
+
+    gst_object_unref (window);
+  }
 
   g_atomic_int_set (&gl_sink->to_quit, 1);
-
-  gst_object_unref (window);
 }
 
 static gboolean
@@ -2477,6 +2530,7 @@ gst_glimage_sink_redisplay (GstGLImageSink * gl_sink)
   GstGLWindow *window;
   GstBuffer *old_stored_buffer[2], *old_sync;
   gulong handler_id;
+  gboolean resize_window = gl_sink->need_resize_window;
 
   window = gst_gl_context_get_window (gl_sink->context);
   if (!window)
@@ -2498,6 +2552,12 @@ gst_glimage_sink_redisplay (GstGLImageSink * gl_sink)
       return FALSE;
     }
 
+    resize_window = TRUE;
+  }
+
+  if (resize_window) {
+    gl_sink->need_resize_window = FALSE;
+
     gst_gl_window_set_preferred_size (window, GST_VIDEO_SINK_WIDTH (gl_sink),
         GST_VIDEO_SINK_HEIGHT (gl_sink));
     gst_gl_window_show (window);
