@@ -39,7 +39,7 @@
  * |[
  * gst-launch-1.0 -v videotestsrc pattern=ball name=left \
  *     videotestsrc name=right glstereomix name=mix \
- *     left. ! vid/x-raw,width=640,height=480! glupload ! mix.  \
+ *     left. ! video/x-raw,width=640,height=480! glupload ! mix.  \
  *     right. ! video/x-raw,width=640,height=480! glupload ! mix.  \
  *     mix. ! video/x-raw'(memory:GLMemory)',multiview-mode=side-by-side ! \
  *     queue ! glimagesink output-multiview-mode=side-by-side
@@ -61,7 +61,9 @@
 #include "config.h"
 #endif
 
+#include "gstglelements.h"
 #include "gstglstereomix.h"
+#include <gst/gl/gl.h>
 
 #define GST_CAT_DEFAULT gst_gl_stereo_mix_debug
 GST_DEBUG_CATEGORY (gst_gl_stereo_mix_debug);
@@ -85,6 +87,8 @@ static void gst_gl_stereo_mix_child_proxy_init (gpointer g_iface,
 G_DEFINE_TYPE_WITH_CODE (GstGLStereoMix, gst_gl_stereo_mix, GST_TYPE_GL_MIXER,
     G_IMPLEMENT_INTERFACE (GST_TYPE_CHILD_PROXY,
         gst_gl_stereo_mix_child_proxy_init));
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (glstereomix, "glstereomix",
+    GST_RANK_NONE, GST_TYPE_GL_STEREO_MIX, gl_element_init (plugin));
 
 static GstCaps *_update_caps (GstVideoAggregator * vagg, GstCaps * caps);
 static gboolean _negotiated_caps (GstAggregator * aggregator, GstCaps * caps);
@@ -641,6 +645,7 @@ _negotiated_caps (GstAggregator * agg, GstCaps * caps)
       GST_GL_TEXTURE_TARGET_2D_STR, NULL);
 
   gst_gl_view_convert_set_caps (mix->viewconvert, in_caps, caps);
+  gst_caps_unref (in_caps);
 
   return TRUE;
 }
