@@ -23,10 +23,9 @@
  * @see_also: vorbisenc, oggdemux
  *
  * This element decodes a Vorbis stream to raw float audio.
- * <ulink url="http://www.vorbis.com/">Vorbis</ulink> is a royalty-free
- * audio codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
- * Foundation</ulink>. As it outputs raw float audio you will often need to
- * put an audioconvert element after it.
+ * [Vorbis](http://www.vorbis.com/) is a royalty-free audio codec maintained
+ * by the [Xiph.org Foundation](http://www.xiph.org/). As it outputs raw float
+ * audio you will often need to put an audioconvert element after it.
  *
  * ## Example pipelines
  * |[
@@ -45,13 +44,14 @@
 #include <gst/audio/audio.h>
 #include <gst/tag/tag.h>
 
+#include "gstvorbiselements.h"
 #include "gstvorbiscommon.h"
 
 #ifndef TREMOR
-GST_DEBUG_CATEGORY_EXTERN (vorbisdec_debug);
+GST_DEBUG_CATEGORY_STATIC (vorbisdec_debug);
 #define GST_CAT_DEFAULT vorbisdec_debug
 #else
-GST_DEBUG_CATEGORY_EXTERN (ivorbisdec_debug);
+GST_DEBUG_CATEGORY_STATIC (ivorbisdec_debug);
 #define GST_CAT_DEFAULT ivorbisdec_debug
 #endif
 
@@ -70,6 +70,19 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 
 #define gst_vorbis_dec_parent_class parent_class
 G_DEFINE_TYPE (GstVorbisDec, gst_vorbis_dec, GST_TYPE_AUDIO_DECODER);
+#ifndef TREMOR
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (vorbisdec, "vorbisdec",
+    GST_RANK_PRIMARY, GST_TYPE_VORBIS_DEC,
+    GST_DEBUG_CATEGORY_INIT (vorbisdec_debug, "vorbisdec", 0,
+        "vorbis decoding element");
+    vorbis_element_init (plugin));
+#else
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (ivorbisdec, "ivorbisdec",
+    GST_RANK_SECONDARY, GST_TYPE_VORBIS_DEC,
+    GST_DEBUG_CATEGORY_INIT (ivorbisdec_debug, "ivorbisdec", 0,
+        "vorbis decoding element (integer decoder)");
+    vorbis_element_init (plugin));
+#endif
 
 static void vorbis_dec_finalize (GObject * object);
 
@@ -191,7 +204,7 @@ vorbis_handle_identification_packet (GstVorbisDec * vd)
     }
     default:{
       GstAudioChannelPosition position[64];
-      gint i, max_pos = MAX (vd->vi.channels, 64);
+      gint i, max_pos = MIN (vd->vi.channels, 64);
 
       GST_ELEMENT_WARNING (vd, STREAM, DECODE,
           (NULL), ("Using NONE channel layout for more than 8 channels"));
@@ -342,7 +355,7 @@ vorbis_handle_header_packet (GstVorbisDec * vd, ogg_packet * packet)
       break;
     default:
       /* ignore */
-      g_warning ("unknown vorbis header packet found");
+      GST_WARNING_OBJECT (vd, "unknown vorbis header packet found");
       res = GST_FLOW_OK;
       break;
   }
@@ -358,6 +371,7 @@ header_read_error:
   }
 }
 
+/* Does not take ownership of buffer */
 static GstFlowReturn
 vorbis_dec_handle_header_buffer (GstVorbisDec * vd, GstBuffer * buffer)
 {
@@ -615,17 +629,20 @@ check_pending_headers (GstVorbisDec * vd)
   /* All good, let's reset ourselves and process the headers */
   vorbis_dec_reset ((GstAudioDecoder *) vd);
   result = vorbis_dec_handle_header_buffer (vd, buffer1);
+  gst_buffer_unref (buffer1);
   if (result != GST_FLOW_OK) {
     gst_buffer_unref (buffer3);
     gst_buffer_unref (buffer5);
     return result;
   }
   result = vorbis_dec_handle_header_buffer (vd, buffer3);
+  gst_buffer_unref (buffer3);
   if (result != GST_FLOW_OK) {
     gst_buffer_unref (buffer5);
     return result;
   }
   result = vorbis_dec_handle_header_buffer (vd, buffer5);
+  gst_buffer_unref (buffer5);
 
   return result;
 
@@ -680,7 +697,11 @@ vorbis_dec_handle_frame (GstAudioDecoder * dec, GstBuffer * buffer)
 
   /* switch depending on packet type */
   if ((gst_ogg_packet_data (packet))[0] & 1) {
+    gboolean have_all_headers;
+
     GST_LOG_OBJECT (vd, "storing header for later analyzis");
+
+    /* An identification packet starts a new set of headers */
     if (vd->pending_headers && (gst_ogg_packet_data (packet))[0] == 0x01) {
       GST_DEBUG_OBJECT (vd,
           "got new identification header packet, discarding old pending headers");
@@ -689,9 +710,29 @@ vorbis_dec_handle_frame (GstAudioDecoder * dec, GstBuffer * buffer)
       vd->pending_headers = NULL;
     }
 
-    vd->pending_headers =
-        g_list_append (vd->pending_headers, gst_buffer_ref (buffer));
-    result = gst_audio_decoder_finish_frame (GST_AUDIO_DECODER (vd), NULL, 1);
+    /* if we have more than 3 headers with the new one and the new one is the
+     * type header, we can initialize the decoder now */
+    have_all_headers = g_list_length (vd->pending_headers) >= 2
+        && (gst_ogg_packet_data (packet))[0] == 0x05;
+
+    if (!vd->pending_headers && (gst_ogg_packet_data (packet))[0] != 0x01) {
+      if (vd->initialized) {
+        GST_DEBUG_OBJECT (vd,
+            "Got another non-identification header after initialization, ignoring");
+      } else {
+        GST_WARNING_OBJECT (vd,
+            "First header was not a identification header, dropping");
+      }
+      result = gst_audio_decoder_finish_frame (GST_AUDIO_DECODER (vd), NULL, 1);
+    } else {
+      vd->pending_headers =
+          g_list_append (vd->pending_headers, gst_buffer_ref (buffer));
+      result = gst_audio_decoder_finish_frame (GST_AUDIO_DECODER (vd), NULL, 1);
+    }
+
+    if (result == GST_FLOW_OK && have_all_headers) {
+      result = check_pending_headers (vd);
+    }
   } else {
     GstClockTime timestamp, duration;
 
@@ -733,11 +774,9 @@ empty_header:
 static void
 vorbis_dec_flush (GstAudioDecoder * dec, gboolean hard)
 {
-#ifdef HAVE_VORBIS_SYNTHESIS_RESTART
   GstVorbisDec *vd = GST_VORBIS_DEC (dec);
 
   vorbis_synthesis_restart (&vd->vd);
-#endif
 }
 
 static void
