@@ -868,9 +868,6 @@ gst_app_sink_setcaps (GstBaseSink * sink, GstCaps * caps)
   g_mutex_lock (&priv->mutex);
   GST_DEBUG_OBJECT (appsink, "receiving CAPS");
 
-  gst_queue_array_push_tail (priv->queue, gst_event_new_caps (caps));
-  gst_queue_status_info_push_event (&priv->queue_status_info);
-
   if (!priv->preroll_buffer)
     gst_caps_replace (&priv->preroll_caps, caps);
   g_mutex_unlock (&priv->mutex);
