@@ -1124,15 +1124,6 @@ gst_audio_base_sink_wait_event (GstBaseSink * bsink, GstEvent * event)
       /* Make sure the ringbuffer will start again if interrupted during event_wait() */
       g_atomic_int_set (&sink->eos_rendering, 1);
       clear_force_start_flag = TRUE;
-
-      /* For gap events, don't actually wait for the clock to
-       * reach that time, or it will drain the ringbuffer, just
-       * ensure we're prerolled and let the next actual buffer
-       * get rendered where it belongs */
-      if (GST_EVENT_TYPE (event) == GST_EVENT_GAP) {
-        ret = gst_base_sink_do_preroll (bsink, GST_MINI_OBJECT_CAST (event));
-        goto done;
-      }
       break;
     default:
       break;
@@ -1271,7 +1262,7 @@ gst_audio_base_sink_custom_slaving (GstAudioBaseSink * sink,
 {
   GstClockTime cinternal, cexternal, crate_num, crate_denom;
   GstClockTime etime, itime;
-  GstClockTimeDiff requested_skew;
+  GstClockTimeDiff requested_skew, drift;
   gint driftsamples;
   gint64 last_align;
 
@@ -1309,10 +1300,18 @@ gst_audio_base_sink_custom_slaving (GstAudioBaseSink * sink,
   }
 
   if (requested_skew > 0) {
-    cexternal = (cexternal > requested_skew) ? (cexternal - requested_skew) : 0;
+    /* Move the external time backward by the requested skew, but don't ever
+     * go negative. Moving the requested skew by the same distance defines
+     * the new clock skew window center point. This allows the clock to
+     * drift equally into either direction after the correction. */
+    if (G_LIKELY (cexternal > requested_skew))
+      drift = requested_skew;
+    else
+      drift = cexternal;
 
-    driftsamples =
-        (sink->ringbuffer->spec.info.rate * requested_skew) / GST_SECOND;
+    cexternal -= drift;
+
+    driftsamples = (sink->ringbuffer->spec.info.rate * drift) / GST_SECOND;
     last_align = sink->priv->last_align;
 
     /* if we were aligning in the wrong direction or we aligned more than what we
@@ -1327,10 +1326,10 @@ gst_audio_base_sink_custom_slaving (GstAudioBaseSink * sink,
     gst_clock_set_calibration (sink->provided_clock, cinternal, cexternal,
         crate_num, crate_denom);
   } else if (requested_skew < 0) {
-    cexternal += ABS (requested_skew);
+    drift = -requested_skew;
+    cexternal += drift;
 
-    driftsamples =
-        (sink->ringbuffer->spec.info.rate * ABS (requested_skew)) / GST_SECOND;
+    driftsamples = (sink->ringbuffer->spec.info.rate * drift) / GST_SECOND;
     last_align = sink->priv->last_align;
 
     /* if we were aligning in the wrong direction or we aligned more than what we
@@ -1728,6 +1727,7 @@ flushing:
   }
 }
 
+#define ABSDIFF(a, b) ((a) > (b) ? (a) - (b) : (b) - (a))
 static gint64
 gst_audio_base_sink_get_alignment (GstAudioBaseSink * sink,
     GstClockTime sample_offset)
@@ -1764,10 +1764,16 @@ gst_audio_base_sink_get_alignment (GstAudioBaseSink * sink,
     if (sink->priv->discont_wait > 0) {
       GstClockTime time = gst_util_uint64_scale_int (sample_offset,
           GST_SECOND, rate);
+      GstClockTime expected_time = gst_util_uint64_scale_int (sink->next_sample,
+          GST_SECOND, rate);
+
       if (sink->priv->discont_time == -1) {
-        /* discont candidate */
-        sink->priv->discont_time = time;
-      } else if (time - sink->priv->discont_time >= sink->priv->discont_wait) {
+        if (ABSDIFF (expected_time, time) >= sink->priv->discont_wait)
+          discont = TRUE;
+        else
+          sink->priv->discont_time = expected_time;
+      } else if (ABSDIFF (time,
+              sink->priv->discont_time) >= sink->priv->discont_wait) {
         /* discont_wait expired, discontinuity detected */
         discont = TRUE;
         sink->priv->discont_time = -1;
@@ -1805,6 +1811,8 @@ gst_audio_base_sink_get_alignment (GstAudioBaseSink * sink,
   return align;
 }
 
+#undef ABSDIFF
+
 static GstFlowReturn
 gst_audio_base_sink_render (GstBaseSink * bsink, GstBuffer * buf)
 {
