@@ -51,7 +51,6 @@ __gst_audio_element_proxy_caps (GstElement * element, GstCaps * templ_caps,
       const GstStructure *caps_s = gst_caps_get_structure (caps, j);
       const GValue *val;
       GstStructure *s;
-      GstCaps *tmp = gst_caps_new_empty ();
 
       s = gst_structure_new_id_empty (q_name);
       if ((val = gst_structure_get_value (caps_s, "rate")))
@@ -61,9 +60,8 @@ __gst_audio_element_proxy_caps (GstElement * element, GstCaps * templ_caps,
       if ((val = gst_structure_get_value (caps_s, "channels-mask")))
         gst_structure_set_value (s, "channels-mask", val);
 
-      gst_caps_append_structure_full (tmp, s,
+      result = gst_caps_merge_structure_full (result, s,
           gst_caps_features_copy (features));
-      result = gst_caps_merge (result, tmp);
     }
   }
 
@@ -128,13 +126,15 @@ __gst_audio_element_proxy_getcaps (GstElement * element, GstPad * sinkpad,
 
   filter_caps = __gst_audio_element_proxy_caps (element, templ_caps, allowed);
 
-  fcaps = gst_caps_intersect (filter_caps, templ_caps);
+  fcaps = gst_caps_intersect_full (filter_caps, templ_caps,
+      GST_CAPS_INTERSECT_FIRST);
   gst_caps_unref (filter_caps);
   gst_caps_unref (templ_caps);
 
   if (filter) {
     GST_LOG_OBJECT (element, "intersecting with %" GST_PTR_FORMAT, filter);
-    filter_caps = gst_caps_intersect (fcaps, filter);
+    filter_caps = gst_caps_intersect_full (fcaps, filter,
+        GST_CAPS_INTERSECT_FIRST);
     gst_caps_unref (fcaps);
     fcaps = filter_caps;
   }
@@ -218,14 +218,16 @@ exit:
 }
 
 #ifdef G_OS_WIN32
+typedef HANDLE (WINAPI * AvSetMmThreadCharacteristicsPtr) (LPCSTR, LPDWORD);
+typedef BOOL (WINAPI * AvRevertMmThreadCharacteristicsPtr) (HANDLE);
+
 /* *INDENT-OFF* */
 static struct
 {
   HMODULE dll;
-  gboolean tried_loading;
 
-  FARPROC AvSetMmThreadCharacteristics;
-  FARPROC AvRevertMmThreadCharacteristics;
+  AvSetMmThreadCharacteristicsPtr AvSetMmThreadCharacteristics;
+  AvRevertMmThreadCharacteristicsPtr AvRevertMmThreadCharacteristics;
 } _gst_audio_avrt_tbl = { 0 };
 /* *INDENT-ON* */
 #endif
@@ -234,29 +236,47 @@ static gboolean
 __gst_audio_init_thread_priority (void)
 {
 #ifdef G_OS_WIN32
-  if (_gst_audio_avrt_tbl.tried_loading)
-    return _gst_audio_avrt_tbl.dll != NULL;
+  static gsize init_once = 0;
+  static gboolean ret = FALSE;
 
-  if (!_gst_audio_avrt_tbl.dll)
-#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP) && !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
-    _gst_audio_avrt_tbl.dll = LoadPackagedLibrary (TEXT ("avrt.dll"), 0);
-#else
+  if (g_once_init_enter (&init_once)) {
+#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
     _gst_audio_avrt_tbl.dll = LoadLibrary (TEXT ("avrt.dll"));
-#endif
 
-  if (!_gst_audio_avrt_tbl.dll) {
-    GST_WARNING ("Failed to set thread priority, can't find avrt.dll");
-    _gst_audio_avrt_tbl.tried_loading = TRUE;
-    return FALSE;
+    if (!_gst_audio_avrt_tbl.dll) {
+      GST_WARNING ("Failed to set thread priority, can't find avrt.dll");
+      goto done;
+    }
+
+    _gst_audio_avrt_tbl.AvSetMmThreadCharacteristics =
+        (AvSetMmThreadCharacteristicsPtr)
+        GetProcAddress (_gst_audio_avrt_tbl.dll,
+        "AvSetMmThreadCharacteristicsA");
+    if (!_gst_audio_avrt_tbl.AvSetMmThreadCharacteristics) {
+      GST_WARNING ("Cannot load AvSetMmThreadCharacteristicsA symbol");
+      FreeLibrary (_gst_audio_avrt_tbl.dll);
+      goto done;
+    }
+
+    _gst_audio_avrt_tbl.AvRevertMmThreadCharacteristics =
+        (AvRevertMmThreadCharacteristicsPtr)
+        GetProcAddress (_gst_audio_avrt_tbl.dll,
+        "AvRevertMmThreadCharacteristics");
+
+    if (!_gst_audio_avrt_tbl.AvRevertMmThreadCharacteristics) {
+      GST_WARNING ("Cannot load AvRevertMmThreadCharacteristics symbol");
+      FreeLibrary (_gst_audio_avrt_tbl.dll);
+      goto done;
+    }
+
+    ret = TRUE;
+
+  done:
+#endif
+    g_once_init_leave (&init_once, 1);
   }
 
-  _gst_audio_avrt_tbl.AvSetMmThreadCharacteristics =
-      GetProcAddress (_gst_audio_avrt_tbl.dll, "AvSetMmThreadCharacteristicsA");
-  _gst_audio_avrt_tbl.AvRevertMmThreadCharacteristics =
-      GetProcAddress (_gst_audio_avrt_tbl.dll,
-      "AvRevertMmThreadCharacteristics");
-
-  _gst_audio_avrt_tbl.tried_loading = TRUE;
+  return ret;
 #endif
 
   return TRUE;
@@ -266,20 +286,54 @@ __gst_audio_init_thread_priority (void)
  * Increases the priority of the thread it's called from
  */
 gboolean
-__gst_audio_set_thread_priority (void)
+__gst_audio_set_thread_priority (gpointer * handle)
 {
 #ifdef G_OS_WIN32
   DWORD taskIndex = 0;
 #endif
 
+  g_return_val_if_fail (handle != NULL, FALSE);
+
+  *handle = NULL;
+
   if (!__gst_audio_init_thread_priority ())
     return FALSE;
 
 #ifdef G_OS_WIN32
-  /* This is only used from ringbuffer thread functions, so we don't need to
-   * ever need to revert the thread priorities. */
-  return _gst_audio_avrt_tbl.AvSetMmThreadCharacteristics (TEXT ("Pro Audio"),
-      &taskIndex) != 0;
+  /* This is only used from ringbuffer thread functions */
+  *handle = (gpointer)
+      _gst_audio_avrt_tbl.AvSetMmThreadCharacteristics (TEXT ("Pro Audio"),
+      &taskIndex);
+  if (*handle == 0) {
+    gchar *errorMsg = g_win32_error_message (GetLastError ());
+
+    GST_WARNING
+        ("Failed to set thread priority, AvSetMmThreadCharacteristics returned: %s",
+        errorMsg);
+    g_free (errorMsg);
+  }
+
+  return *handle != 0;
+#else
+  return TRUE;
+#endif
+}
+
+/*
+ * Restores the priority of the thread that was increased
+ * with __gst_audio_set_thread_priority.
+ * This function must be called from the same thread that called the
+ * __gst_audio_set_thread_priority function.
+ * See https://docs.microsoft.com/en-us/windows/win32/api/avrt/nf-avrt-avsetmmthreadcharacteristicsw#remarks
+ */
+gboolean
+__gst_audio_restore_thread_priority (gpointer handle)
+{
+#ifdef G_OS_WIN32
+  if (!handle)
+    return FALSE;
+
+  return _gst_audio_avrt_tbl.AvRevertMmThreadCharacteristics ((HANDLE) handle);
 #else
   return TRUE;
 #endif
