@@ -88,6 +88,7 @@ GstEGLImage *           gst_egl_image_from_dmabuf               (GstGLContext *
                                                                  const GstVideoInfo * in_info,
                                                                  gint plane,
                                                                  gsize offset);
+
 GST_GL_API
 GstEGLImage *           gst_egl_image_from_dmabuf_direct        (GstGLContext * context,
                                                                  gint *fd,
@@ -101,6 +102,14 @@ GstEGLImage *           gst_egl_image_from_dmabuf_direct_target (GstGLContext *
                                                                  GstGLTextureTarget target);
 
 GST_GL_API
+GstEGLImage *           gst_egl_image_from_dmabuf_direct_target_with_dma_drm
+                                                                (GstGLContext * context,
+                                                                 guint n_planes,
+                                                                 gint * fd,
+                                                                 const gsize * offset,
+                                                                 const GstVideoInfoDmaDrm * in_info_dma,
+                                                                 GstGLTextureTarget target);
+GST_GL_API
 gboolean                gst_egl_image_export_dmabuf             (GstEGLImage *image, int *fd, gint *stride, gsize *offset);
 #endif
 
