@@ -50,7 +50,7 @@ G_DEFINE_TYPE (GstDiscovererStreamInfo, gst_discoverer_stream_info,
 static void
 gst_discoverer_stream_info_init (GstDiscovererStreamInfo * info)
 {
-  /* Nothing needs initialization */
+  info->stream_number = -1;
 }
 
 static void
@@ -74,6 +74,8 @@ gst_discoverer_stream_info_finalize (GObject * object)
 
   if (info->misc)
     gst_structure_free (info->misc);
+
+  G_OBJECT_CLASS (gst_discoverer_stream_info_parent_class)->finalize (object);
 }
 
 static void
@@ -143,6 +145,8 @@ gst_discoverer_info_copy_int (GstDiscovererStreamInfo * info,
   if (stream_map)
     g_hash_table_insert (stream_map, info, ret);
 
+  ret->stream_number = info->stream_number;
+
   return ret;
 }
 
@@ -175,7 +179,11 @@ gst_discoverer_container_info_finalize (GObject * object)
 
   gst_discoverer_stream_info_list_free (info->streams);
 
-  gst_discoverer_stream_info_finalize ((GObject *) info);
+  if (info->tags)
+    gst_tag_list_unref (info->tags);
+
+  G_OBJECT_CLASS (gst_discoverer_container_info_parent_class)->finalize
+      (object);
 }
 
 static void
@@ -204,6 +212,9 @@ gst_stream_container_info_copy_int (GstDiscovererContainerInfo * ptr,
       g_hash_table_insert (stream_map, tmp->data, subtop);
   }
 
+  if (ptr->tags)
+    ret->tags = gst_tag_list_copy (ptr->tags);
+
   return ret;
 }
 
@@ -354,7 +365,12 @@ G_DEFINE_TYPE (GstDiscovererInfo, gst_discoverer_info, G_TYPE_OBJECT);
 static void
 gst_discoverer_info_init (GstDiscovererInfo * info)
 {
+#if GLIB_CHECK_VERSION(2,74,0)
+  info->missing_elements_details =
+      g_ptr_array_new_null_terminated (16, g_free, TRUE);
+#else
   info->missing_elements_details = g_ptr_array_new_with_free_func (g_free);
+#endif
 }
 
 static void
@@ -380,6 +396,8 @@ gst_discoverer_info_finalize (GObject * object)
   g_free (info->cachefile);
 
   g_ptr_array_unref (info->missing_elements_details);
+
+  G_OBJECT_CLASS (gst_discoverer_info_parent_class)->finalize (object);
 }
 
 static GstDiscovererInfo *
@@ -437,6 +455,14 @@ gst_discoverer_info_copy (GstDiscovererInfo * ptr)
   if (ptr->toc)
     ret->toc = gst_toc_ref (ptr->toc);
 
+  if (ptr->missing_elements_details->len > 0) {
+    guint i;
+
+    for (i = 0; i < ptr->missing_elements_details->len; i++)
+      g_ptr_array_add (ret->missing_elements_details,
+          g_strdup (ptr->missing_elements_details->pdata[i]));
+  }
+
   g_hash_table_destroy (stream_map);
   return ret;
 }
@@ -497,7 +523,7 @@ gst_discoverer_info_get_streams (GstDiscovererInfo * info, GType streamtype)
  *
  * Finds all the #GstDiscovererAudioInfo contained in @info
  *
- * Returns: (transfer full) (element-type GstPbutils.DiscovererStreamInfo): A #GList of
+ * Returns: (transfer full) (element-type GstPbutils.DiscovererAudioInfo): A #GList of
  * matching #GstDiscovererStreamInfo. The caller should free it with
  * gst_discoverer_stream_info_list_free().
  */
@@ -513,7 +539,7 @@ gst_discoverer_info_get_audio_streams (GstDiscovererInfo * info)
  *
  * Finds all the #GstDiscovererVideoInfo contained in @info
  *
- * Returns: (transfer full) (element-type GstPbutils.DiscovererStreamInfo): A #GList of
+ * Returns: (transfer full) (element-type GstPbutils.DiscovererVideoInfo): A #GList of
  * matching #GstDiscovererStreamInfo. The caller should free it with
  * gst_discoverer_stream_info_list_free().
  */
@@ -529,7 +555,7 @@ gst_discoverer_info_get_video_streams (GstDiscovererInfo * info)
  *
  * Finds all the #GstDiscovererSubtitleInfo contained in @info
  *
- * Returns: (transfer full) (element-type GstPbutils.DiscovererStreamInfo): A #GList of
+ * Returns: (transfer full) (element-type GstPbutils.DiscovererSubtitleInfo): A #GList of
  * matching #GstDiscovererStreamInfo. The caller should free it with
  * gst_discoverer_stream_info_list_free().
  */
@@ -546,7 +572,7 @@ gst_discoverer_info_get_subtitle_streams (GstDiscovererInfo * info)
  *
  * Finds all the #GstDiscovererContainerInfo contained in @info
  *
- * Returns: (transfer full) (element-type GstPbutils.DiscovererStreamInfo): A #GList of
+ * Returns: (transfer full) (element-type GstPbutils.DiscovererContainerInfo): A #GList of
  * matching #GstDiscovererStreamInfo. The caller should free it with
  * gst_discoverer_stream_info_list_free().
  */
@@ -596,7 +622,7 @@ gst_discoverer_stream_info_get_stream_type_nick (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_previous:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer full): the previous #GstDiscovererStreamInfo in a chain.
+ * Returns: (transfer full) (nullable): the previous #GstDiscovererStreamInfo in a chain.
  * %NULL for starting points. Unref with #gst_discoverer_stream_info_unref
  * after usage.
  */
@@ -614,7 +640,7 @@ gst_discoverer_stream_info_get_previous (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_next:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer full): the next #GstDiscovererStreamInfo in a chain. %NULL
+ * Returns: (transfer full) (nullable): the next #GstDiscovererStreamInfo in a chain. %NULL
  * for final streams.
  * Unref with #gst_discoverer_stream_info_unref after usage.
  */
@@ -633,7 +659,7 @@ gst_discoverer_stream_info_get_next (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_caps:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer full): the #GstCaps of the stream. Unref with
+ * Returns: (transfer full) (nullable): the #GstCaps of the stream. Unref with
  * #gst_caps_unref after usage.
  */
 GstCaps *
@@ -650,7 +676,7 @@ gst_discoverer_stream_info_get_caps (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_tags:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer none): the tags contained in this stream. If you wish to
+ * Returns: (transfer none) (nullable): the tags contained in this stream. If you wish to
  * use the tags after the life-time of @info you will need to copy them.
  */
 const GstTagList *
@@ -665,7 +691,7 @@ gst_discoverer_stream_info_get_tags (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_toc:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer none): the TOC contained in this stream. If you wish to
+ * Returns: (transfer none) (nullable): the TOC contained in this stream. If you wish to
  * use the TOC after the life-time of @info you will need to copy it.
  */
 const GstToc *
@@ -680,7 +706,7 @@ gst_discoverer_stream_info_get_toc (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_stream_id:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: (transfer none): the stream ID of this stream. If you wish to
+ * Returns: (transfer none) (nullable): the stream ID of this stream. If you wish to
  * use the stream ID after the life-time of @info you will need to copy it.
  */
 const gchar *
@@ -699,7 +725,7 @@ gst_discoverer_stream_info_get_stream_id (GstDiscovererStreamInfo * info)
  * Deprecated: This functions is deprecated since version 1.4, use
  * #gst_discoverer_info_get_missing_elements_installer_details
  *
- * Returns: (transfer none): additional information regarding the stream (for
+ * Returns: (transfer none) (nullable): additional information regarding the stream (for
  * example codec version, profile, etc..). If you wish to use the #GstStructure
  * after the life-time of @info you will need to copy it.
  */
@@ -712,6 +738,23 @@ gst_discoverer_stream_info_get_misc (GstDiscovererStreamInfo * info)
 }
 #endif
 
+/**
+ * gst_discoverer_stream_info_get_stream_number:
+ * @info: a #GstDiscovererStreamInfo
+ *
+ * Returns: the stream number, -1 if no index could be determined. This property
+ * acts as a unique identifier as a 'int' for the stream.
+ *
+ * Since: 1.20
+ */
+gint
+gst_discoverer_stream_info_get_stream_number (GstDiscovererStreamInfo * info)
+{
+  g_return_val_if_fail (GST_IS_DISCOVERER_STREAM_INFO (info), -1);
+
+  return info->stream_number;
+}
+
 /* GstDiscovererContainerInfo */
 
 /**
@@ -738,6 +781,23 @@ gst_discoverer_container_info_get_streams (GstDiscovererContainerInfo * info)
   return res;
 }
 
+/**
+ * gst_discoverer_container_info_get_tags:
+ * @info: a #GstDiscovererStreamInfo
+ *
+ * Returns: (transfer none) (nullable): tags specific to the given container. If you wish to use
+ * the tags after the life-time of @info, you will need to copy them.
+ *
+ * Since: 1.20
+ */
+const GstTagList *
+gst_discoverer_container_info_get_tags (const GstDiscovererContainerInfo * info)
+{
+  g_return_val_if_fail (GST_IS_DISCOVERER_CONTAINER_INFO (info), NULL);
+
+  return info->tags;
+}
+
 /* GstDiscovererAudioInfo */
 
 #define AUDIO_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
@@ -807,7 +867,7 @@ AUDIO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
  * gst_discoverer_audio_info_get_language:
  * @info: a #GstDiscovererAudioInfo
  *
- * Returns: the language of the stream, or NULL if unknown.
+ * Returns: (nullable): the language of the stream, or NULL if unknown.
  */
 
 AUDIO_INFO_ACCESSOR_CODE (language, const gchar *, NULL);
@@ -940,7 +1000,7 @@ gst_discoverer_video_info_is_image (const GstDiscovererVideoInfo * info)
  * gst_discoverer_subtitle_info_get_language:
  * @info: a #GstDiscovererSubtitleInfo
  *
- * Returns: the language of the stream, or NULL if unknown.
+ * Returns: (nullable): the language of the stream, or NULL if unknown.
  */
 
 SUBTITLE_INFO_ACCESSOR_CODE (language, const gchar *, NULL);
@@ -975,7 +1035,7 @@ DISCOVERER_INFO_ACCESSOR_CODE (result, GstDiscovererResult, GST_DISCOVERER_OK);
  * gst_discoverer_info_get_stream_info:
  * @info: a #GstDiscovererInfo
  *
- * Returns: (transfer full): the structure (or topology) of the URI as a
+ * Returns: (transfer full) (nullable): the structure (or topology) of the URI as a
  * #GstDiscovererStreamInfo.
  * This structure can be traversed to see the original hierarchy. Unref with
  * gst_discoverer_stream_info_unref() after usage.
@@ -1051,29 +1111,31 @@ DISCOVERER_INFO_ACCESSOR_CODE (live, gboolean, FALSE);
  * Deprecated: This functions is deprecated since version 1.4, use
  * #gst_discoverer_info_get_missing_elements_installer_details
  *
- * Returns: (transfer none): Miscellaneous information stored as a #GstStructure
+ * Returns: (transfer none) (nullable): Miscellaneous information stored as a #GstStructure
  * (for example: information about missing plugins). If you wish to use the
  * #GstStructure after the life-time of @info, you will need to copy it.
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (misc, const GstStructure *, NULL);
-#endif
 
 /**
  * gst_discoverer_info_get_tags:
  * @info: a #GstDiscovererInfo
  *
- * Returns: (transfer none): all tags contained in the URI. If you wish to use
+ * Returns: (transfer none) (nullable): all tags contained in the URI. If you wish to use
  * the tags after the life-time of @info, you will need to copy them.
+ *
+ * Deprecated: 1.20: Use gst_discoverer_{container,stream}_info_get_tags() instead.
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (tags, const GstTagList *, NULL);
+#endif
 
 /**
  * gst_discoverer_info_get_toc:
  * @info: a #GstDiscovererInfo
  *
- * Returns: (transfer none): TOC contained in the URI. If you wish to use
+ * Returns: (transfer none) (nullable): TOC contained in the URI. If you wish to use
  * the TOC after the life-time of @info, you will need to copy it.
  */
 
@@ -1130,7 +1192,6 @@ const gchar **
 gst_discoverer_info_get_missing_elements_installer_details (const
     GstDiscovererInfo * info)
 {
-
   if (info->result != GST_DISCOVERER_MISSING_PLUGINS) {
     GST_WARNING_OBJECT (info, "Trying to get missing element installed details "
         "but result is not 'MISSING_PLUGINS'");
@@ -1138,11 +1199,5 @@ gst_discoverer_info_get_missing_elements_installer_details (const
     return NULL;
   }
 
-  if (info->missing_elements_details->pdata[info->missing_elements_details->
-          len]) {
-    GST_DEBUG ("Adding NULL pointer to the end of missing_elements_details");
-    g_ptr_array_add (info->missing_elements_details, NULL);
-  }
-
   return (const gchar **) info->missing_elements_details->pdata;
 }
