@@ -480,7 +480,7 @@ gst_exif_reader_add_pending_tag (GstExifReader * reader, GstExifTagData * data)
 {
   GstExifTagData *copy;
 
-  copy = g_slice_new (GstExifTagData);
+  copy = g_new (GstExifTagData, 1);
   memcpy (copy, data, sizeof (GstExifTagData));
 
   reader->pending_tags = g_slist_prepend (reader->pending_tags, copy);
@@ -509,7 +509,7 @@ gst_exif_reader_reset (GstExifReader * reader, gboolean return_taglist)
   for (walker = reader->pending_tags; walker; walker = g_slist_next (walker)) {
     GstExifTagData *data = (GstExifTagData *) walker->data;
 
-    g_slice_free (GstExifTagData, data);
+    g_free (data);
   }
   g_slist_free (reader->pending_tags);
 
@@ -1303,9 +1303,13 @@ parse_exif_ascii_tag (GstExifReader * reader, const GstExifTagMatch * tag,
       GstDateTime *d;
 
       d = gst_date_time_new_local_time (year, month, day, hour, minute, second);
-      gst_tag_list_add (reader->taglist, GST_TAG_MERGE_REPLACE,
-          tag->gst_tag, d, NULL);
-      gst_date_time_unref (d);
+      if (d) {
+        gst_tag_list_add (reader->taglist, GST_TAG_MERGE_REPLACE,
+            tag->gst_tag, d, NULL);
+        gst_date_time_unref (d);
+      } else {
+        GST_WARNING ("Failed to parse %s into a datetime tag", utfstr);
+      }
     } else {
       GST_WARNING ("Failed to parse %s into a datetime tag", utfstr);
     }
