@@ -664,6 +664,12 @@ subrip_unescape_formatting (gchar * txt, gconstpointer allowed_tags_ptr,
   res = g_regex_replace (tag_regex, txt, strlen (txt), 0,
       replace_pattern, 0, NULL);
 
+  /* Replacing can fail. Return an empty string in that case. */
+  if (!res) {
+    strcpy (txt, "");
+    return;
+  }
+
   /* res will always be shorter than the input or identical, so this
    * copy is OK */
   strcpy (txt, res);
@@ -848,7 +854,7 @@ parse_subrip_time (const gchar * ts_string, GstClockTime * t)
   g_strdelimit (s, " ", '0');
   g_strdelimit (s, ".", ',');
 
-  /* make sure we have exactly three digits after he comma */
+  /* make sure we have exactly three digits after the comma */
   p = strchr (s, ',');
   if (p == NULL) {
     /* If there isn't a ',' the timestamp is broken */
@@ -857,6 +863,15 @@ parse_subrip_time (const gchar * ts_string, GstClockTime * t)
     return FALSE;
   }
 
+  /* Check if the comma is too far into the string to avoid
+   * stack overflow when zero-padding the sub-second part.
+   *
+   * Allow for 3 digits of hours just in case. */
+  if ((p - s) > sizeof ("hhh:mm:ss,")) {
+    GST_WARNING ("failed to parse subrip timestamp string '%s'", s);
+    return FALSE;
+  }
+
   ++p;
   len = strlen (p);
   if (len > 3) {
@@ -1035,6 +1050,10 @@ parse_subrip (ParserState * state, const gchar * line)
         g_string_append_c (state->buf, '\n');
       g_string_append (state->buf, line);
       if (strlen (line) == 0) {
+        if (!g_utf8_validate (state->buf->str, state->buf->len, NULL)) {
+          g_string_truncate (state->buf, 0);
+          return NULL;
+        }
         ret = g_markup_escape_text (state->buf->str, state->buf->len);
         g_string_truncate (state->buf, 0);
         state->state = 0;
