@@ -1039,6 +1039,7 @@ gst_video_rate_sink_event (GstBaseTransform * trans, GstEvent * event)
           videorate->base_ts = segment.start;
         videorate->out_frame_count = 0;
         videorate->next_ts = GST_CLOCK_TIME_NONE;
+        videorate->last_ts = GST_CLOCK_TIME_NONE;
         gst_buffer_replace (&videorate->prevbuf, NULL);
       }
 
@@ -1889,8 +1890,12 @@ gst_video_rate_transform_ip (GstBaseTransform * trans, GstBuffer * buffer)
 
         next_ts = videorate->base_ts - (
             (videorate->base_ts - next_ts) * videorate->rate);
-        next_end_ts = videorate->base_ts - (MAX (0,
-                (videorate->base_ts - next_end_ts)) * videorate->rate);
+        if (videorate->base_ts > next_end_ts)
+          next_end_ts =
+              videorate->base_ts - ((videorate->base_ts -
+                  next_end_ts) * videorate->rate);
+        else
+          next_end_ts = videorate->base_ts;
 
         diff1 = ABSDIFF (prev_endtime, next_end_ts);
         diff2 = ABSDIFF (in_endtime, next_end_ts);
